import requests
import json


class Id64:
    """Splits the 64bit Identifier in its parts"""
    LookupPath = "./SectorLookup.json"
    try:
        file = open(LookupPath, "r")
        Sectors = json.loads(file.read())
        file.close()
    except FileNotFoundError:
        print("File SectorLookup.json not found")
        file.close()
        quit(-1)

    def __init__(self, id: int = 0):
        self. ValidID = True
        self.Layer = id % 2 ** 3; id >>= 3
        a = 7 - self.Layer
        b = 3 * self.Layer + 11
        self.BoxelZ   = id % 2 ** a; id >>= a
        self.SectorZ  = id % 2 ** 7; id >>= 7
        self.BoxelY   = id % 2 ** a; id >>= a 
        self.SectorY  = id % 2 ** 6; id >>= 6
        self.BoxelX   = id % 2 ** a; id >>= a
        self.SectorX  = id % 2 ** 7; id >>= 7
        self.SystemID = id % 2 ** b; id >>= b
        self.BodyID = id % 2 ** 9; id >>= 9
        if id != 0: #if the id is not 0 now, the given number was to big hence not a valid id
            self.ValidID = False

    def getID64(self):
        if self.ValidID == False:
            return None
        else:
            id64 = self.BodyID; id64 <<= (11 + (3 * self.Layer))
            id64 += self.SystemID; id64 <<= 7
            id64 += self.SectorX; id64 <<= 7 - self.Layer
            id64 += self.BoxelX; id64 <<= 6
            id64 += self.SectorY; id64 <<= 7 - self.Layer
            id64 += self.BoxelY; id64 <<= 7
            id64 += self.SectorZ; id64 <<= 7 - self.Layer
            id64 += self.BoxelZ; id64 <<= 3
            id64 += self.Layer
            return id64

    def asTupple(self):
        """gives the parts as tupple"""
        return (self.BodyID, self.SystemID, self.SectorX, self.BoxelX, self.SectorY, self.BoxelY, self.SectorZ, self.BoxelZ, self.Layer)

    def getBoxelKey(self):
        return getBoxelKey(self.BoxelX, self.BoxelY, self.BoxelZ)

    def getSectorKey(self):
        return getSectorKey(self.SectorX, self.SectorY, self.SectorZ)
    
    def getLetterCode(self, Body = 0, Imply = True):
        a = (self.getBoxelKey() // 26 ** 3)
        str_1 = f"{ chr(self.getBoxelKey() % 26 + 65) }{ chr((self.getBoxelKey() // 26) % 26 + 65) }-{ chr((self.getBoxelKey() // 26 ** 2) % 26 + 65) } { chr(self.Layer + 97 ) }"
        str_2 = f"{(Body << (11+(3*self.Layer))) + self.SystemID}"
        if (a == 0 & Imply):
            return str_1 + str_2
        else:
            return str_1 + f"{int(a)}-" + str_2

    def lookupSectorName(self):
        try:
            return Id64.Sectors[f"{self.getSectorKey()}"]["Name"]
        except KeyError:
            return(f"[{self.SectorX}; {self.SectorY}; {self.SectorZ}]")

    def setBodyId(self, Bid: int):
        if 0 <= Bid < 512:
            self.BodyID = Bid
        else:
            print("BodyId must be 0 <= BodyId < 512")

    def getSearchString(self, Body = 0, Imply = 0):
        return self.lookupSectorName() + " " + self.getLetterCode(Body, Imply)

def getBoxelKey(x, y, z):
    return x + y * 128 + z * 128 ** 2

def getSectorKey(X, Y, Z):
    return X + Y * 128 + int(Z) * 128 ** 2

def EDSM_API_getSystemId64ByName(name):
    response = requests.get("https://www.edsm.net/api-system-v1/deaths", {"systemName": name}) #only interested in ID64. death has the lest amount of data
    if response.status_code == 200:
        data = json.loads(response.text)
        #print(data)
        if "id64" in data:
            return Id64(data["id64"])
    return None

#Example
SystemName = "Shinrarta Dezhra"
SD = EDSM_API_getSystemId64ByName(SystemName)
if SD == None:
    print(f"ERROR: \"{SystemName}\" not found in EDSM Database")
else:
    for i in range(0,512):
        SD.BodyID = i
        print(f"{{ \"BodyID\":{i}, \"id64\":{SD.getID64()}, \"SearchString\":\"{SD.getSearchString(i)}\" }}")


